﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NetSDKCS;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.IO;

namespace LatticeScreenDemo
{
    public partial class LatticeScreenDemo : Form
    {
        private static fDisConnectCallBack m_DisConnectCallBack;
        private static fHaveReConnectCallBack m_ReConnectCallBack;
        private const int m_WaitTime = 5000;
        private const int ListViewCount = 100;

        private IntPtr m_LoginID = IntPtr.Zero;
        private NET_DEVICEINFO_Ex m_DeviceInfo;

        public LatticeScreenDemo()
        {
            InitializeComponent();
            this.Load += new EventHandler(LatticeScreenDemo_Load);
        }

        private void LatticeScreenDemo_Load(object sender, EventArgs e)
        {
            m_DisConnectCallBack = new fDisConnectCallBack(DisConnectCallBack);
            m_ReConnectCallBack = new fHaveReConnectCallBack(ReConnectCallBack);
            try
            {
                NETClient.Init(m_DisConnectCallBack, IntPtr.Zero, null);
                NETClient.SetAutoReconnect(m_ReConnectCallBack, IntPtr.Zero);
                InitOrLogoutUI();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                Process.GetCurrentProcess().Kill();
            }
        }

        private void DisConnectCallBack(IntPtr lLoginID, IntPtr pchDVRIP, int nDVRPort, IntPtr dwUser)
        {
            this.BeginInvoke((Action)UpdateDisConnectUI);
        }

        private void UpdateDisConnectUI()
        {
            this.Text = "LatticeScreenDemo(点阵屏Demo) --- Offline(离线)";
        }

        private void ReConnectCallBack(IntPtr lLoginID, IntPtr pchDVRIP, int nDVRPort, IntPtr dwUser)
        {
            this.BeginInvoke((Action)UpdateReConnectUI);
        }
        private void UpdateReConnectUI()
        {
            this.Text = "LatticeScreenDemo(点阵屏Demo) --- Online(在线)";
        }

        private void InitOrLogoutUI()
        {
            login_button.Text = "Login(登录)";
            m_LoginID = IntPtr.Zero;
            this.Text = "LatticeScreenDemo(点阵屏Demo)";
        }

        private void LoginUI()
        {
            this.Text = "LatticeScreenDemo(点阵屏Demo) --- Online(在线)";
            login_button.Text = "Logout(登出)";
            cmb_PassState.SelectedIndex = 0;
        }

        private void port_textBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar != 8 && !Char.IsDigit(e.KeyChar))
            {
                e.Handled = true;
            }
        }

        private void login_button_Click(object sender, EventArgs e)
        {
            if (IntPtr.Zero == m_LoginID)
            {
                ushort port = 0;
                try
                {
                    port = Convert.ToUInt16(port_textBox.Text.Trim());
                }
                catch
                {
                    MessageBox.Show("Input port error(输入端口错误)!");
                    return;
                }
                m_DeviceInfo = new NET_DEVICEINFO_Ex();
                m_LoginID = NETClient.LoginWithHighLevelSecurity(ip_textBox.Text.Trim(), port, name_textBox.Text.Trim(), pwd_textBox.Text.Trim(), EM_LOGIN_SPAC_CAP_TYPE.TCP, IntPtr.Zero, ref m_DeviceInfo);
                if (IntPtr.Zero == m_LoginID)
                {
                    MessageBox.Show(this, NETClient.GetLastError());
                    return;
                }
                LoginUI();
            }
            else
            {
                bool result = NETClient.Logout(m_LoginID);
                if (!result)
                {
                    MessageBox.Show(this, NETClient.GetLastError());
                    return;
                }
                m_LoginID = IntPtr.Zero;
                InitOrLogoutUI();
            }
        }
     
        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);
            NETClient.Cleanup();
        }

        private void cmb_PassState_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmb_PassState.SelectedIndex == 0)
            {
                txt_PlateNum.Enabled = true;
                txt_CarMaster.Enabled = true;
                cmb_UserType.Enabled = true;
                txt_RemainDay.Enabled = true;
                time_In.Enabled = true;
                time_Out.Enabled = true;
                txt_ParkTime.Enabled = true;
                txt_ParkCharge.Enabled = true;
                rb_NoPassage.Enabled = true;
                rb_LetThrough.Enabled = true;
                txt_SubUserType.Enabled = true;
                txt_Remarks.Enabled = true;

                txt_RemainSpace.Enabled = false;
            }

            if (cmb_PassState.SelectedIndex == 1)
            {
                txt_PlateNum.Enabled = false;
                txt_CarMaster.Enabled = false;
                cmb_UserType.Enabled = false;
                txt_RemainDay.Enabled = false;
                time_In.Enabled = false;
                time_Out.Enabled = false;
                txt_ParkTime.Enabled = false;
                txt_ParkCharge.Enabled = false;
                rb_NoPassage.Enabled = false;
                rb_LetThrough.Enabled = false;
                txt_SubUserType.Enabled = false;
                txt_Remarks.Enabled = false;

                txt_RemainSpace.Enabled = true;
            }
        }

        private void btn_Set_Click(object sender, EventArgs e)
        {
            NET_CTRL_SET_PARK_INFO stSetParkInfo = new NET_CTRL_SET_PARK_INFO();
            stSetParkInfo.dwSize = (uint)Marshal.SizeOf(stSetParkInfo);

            try
            {
                int pass_state = cmb_PassState.SelectedIndex;
                stSetParkInfo.emCarStatus = (EM_CARPASS_STATUS)(pass_state + 1);
                stSetParkInfo.szPlateNumber = txt_PlateNum.Text.Trim();
                stSetParkInfo.szMasterofCar = txt_CarMaster.Text.Trim();
                if (cmb_UserType.Enabled)
                {
                    int user_type = cmb_UserType.SelectedIndex;
                    switch (user_type)
                    {
                        case 0:
                            stSetParkInfo.szUserType = "monthlyCardUser";
                            break;
                        case 1:
                            stSetParkInfo.szUserType = "yearlyCardUser";
                            break;
                        case 2:
                            stSetParkInfo.szUserType = "longTimeUser";
                            break;
                        case 3:
                            stSetParkInfo.szUserType = "casualUser";
                            break;
                        default:
                            break;
                    }
                }

                if (txt_RemainDay.Enabled && !string.IsNullOrEmpty(txt_RemainDay.Text.Trim()))
                {
                    stSetParkInfo.nRemainDay = uint.Parse(txt_RemainDay.Text.Trim());
                }
                if (time_In.Enabled)
                {
                    DateTime inTime = new DateTime(time_In.Value.Year, time_In.Value.Month, time_In.Value.Day, time_In.Value.Hour, time_In.Value.Minute, time_In.Value.Second);
                    stSetParkInfo.stuInTime = NET_TIME.FromDateTime(inTime);
                }
                if (time_Out.Enabled)
                {
                    DateTime outTime = new DateTime(time_Out.Value.Year, time_Out.Value.Month, time_Out.Value.Day, time_Out.Value.Hour, time_Out.Value.Minute, time_Out.Value.Second);
                    stSetParkInfo.stuOutTime = NET_TIME.FromDateTime(outTime);
                }
                if (txt_ParkTime.Enabled && !string.IsNullOrEmpty(txt_ParkTime.Text.Trim()))
                {
                    stSetParkInfo.nParkTime = uint.Parse(txt_ParkTime.Text.Trim());
                }
                stSetParkInfo.szParkCharge = txt_ParkCharge.Text.Trim();
                if (txt_RemainSpace.Enabled && !string.IsNullOrEmpty(txt_RemainSpace.Text.Trim()))
                {
                    stSetParkInfo.nRemainSpace = uint.Parse(txt_RemainSpace.Text.Trim());
                }
                if (rb_LetThrough.Enabled && rb_LetThrough.Checked)
                {
                    stSetParkInfo.nPassEnable = 1;
                }
                if (rb_NoPassage.Enabled && rb_NoPassage.Checked)
                {
                    stSetParkInfo.nPassEnable = 0;
                }
                stSetParkInfo.szSubUserType = txt_SubUserType.Text.Trim();
                stSetParkInfo.szRemarks = txt_Remarks.Text.Trim();
                stSetParkInfo.szCustom = txt_Custom.Text.Trim();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }
            


            IntPtr pInfo = Marshal.AllocHGlobal(Marshal.SizeOf(stSetParkInfo));
            Marshal.StructureToPtr(stSetParkInfo, pInfo, true);

            bool bRet = NETClient.ControlDevice(m_LoginID, EM_CtrlType.SET_PARK_INFO, pInfo, 5000);
            if (bRet)
            {
                Console.WriteLine("ControlDevice SET_PARK_INFO success!");
            }
            else
            {
                MessageBox.Show(NETClient.GetLastError());
                Console.WriteLine("ControlDevice SET_PARK_INFO fail!");
            }
        }
    }

}
